//-----------------------------------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------------------------

#ifndef( MA_Screenplay_Temp )
	#declare MA_Screenplay_Temp = version;
	#version 3.7;

	//Here we're still inside an If-Block !! (the first #ifndef)

//-----------------------------------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------------------------


/* ****************************************************************************************************************************

--------------------------------------------------------
MA_Screenplay
--------------------------------------------------------
Purpose:		Help to generate complex animations
Author:			Martin Seydler
Website:		www.maetes.com
Documentation:	https://www.maetes.com/en/povray-os/ma_screenplay
Date:			2018 an later
Version:		1.0 (December 2024)
Copyright:		See above (author), below near any macro (orignal author and sources) and everywhere (www.povray.org)
License:		Use it in your projects as you like. A mention and a link to my website would be nice.




*/// **************************************************************************************************************************



/* ----------------------------------------------------------------------------------------------------------------------------
Declaring some variables

IMPORTANT:
With the exeption of SCR_FPS the word "Second" here does NOT mean an integer-value (0,1,2,3, ...) but a float (0.0, 0.1, 0.2, ...).
Its one point in time of your simulation, so more a millisecond than a second.
If you use FPS of 20, first "Second" is at 0.00 millisecond (ms), 2nd at 0.05 ms, 3rd at 0.10 ms ... and so on.


SCR_Data			Saves all Screenplay-Informations for the actual frame or second/millisecond.
					Its an array of arrays, every row has three items/values: Name of the object, Property-Name, Property-Value
					Example of one array-row: array[3] { "spacecraft/body/solarcell-1", "rotation", "180" }
					The array has 200 rows, increase it if you need more.
					Unfortunately povray doesnt provide functions to resize an array, and its to complex to resize it manually.
					Every, every time needed. See Resize_Array() in arrays.inc (povray standard-include directory).
					So the fastest and most un-dirty way is to use from the start on more dimensions than needed.
					
SCR_DataIndex		Describes the next Index (next row) of SCR_Data to be write into.
					After adding anoter row (with macro SCR_SetNumber() or others), DataIndex will become incremented.
					
SCR_FPS				Frames per Second
					Is needed to calculate "frame_number" into "seconds".
					Dont forget that fps * the longest "mSecond3" in you timeline should bigger or equal "Final_Frame" in the Ini-file
					Example, maybe for testing:	FPS = 10	mSecond3 = 6.0		Final_Frame = 60
					For your final round		FPS = 30	mSecond3 = 6.0		Final_Frame = 180 
					Or set Final_Frame to 10000, the macro will end itself automatically by raising an Error.

SCR_SecondStart		Default = 0.
					Change this if you like to skip some seconds.
					Note that this is for development only, not to create parts of the animations in steps,
					because it will start allways with picname 000.
					If you like to create part with correct picturenames, use Initial_Frame in the Ini-File.
					Alternatively use CMD-Parameter +KFI and +KFF.

SCR_SecondEnd		Last Second of the complete animation.
					The Simulation needs this to calculate the number of Frames,
					and the INI-Call to calculate the last Frame (it ignores the INI-Value Final_Frame)

SCR_SecondFreeze	Second that will be shown, if you CMD-Line does call a single picture instead of creating an animation.
					Use this to check one specified point in time.
							
SCR_RunSimulation	Runs a simulated Animation from SCR_SecondStart to SCR_SecondEnd.
					Most Errors can be found on this way, but only in the definition-section,
					not in the final block, where all objects gets created (this will not touched by the simulation)
					Set it to true/false, false if you dont want to waste time.
					*** not implemented in the examples at the moment ***

SCR_Second3Diff		Value to substract from every Second2, Second3 or SecondEnd-Parameter.
SCR_Second2Diff		If one object is visible from 0 to 10 and the next from 10 to 20, in second 10 both objects are visible.
SCR_SecondEndDiff	Most of the time not welcome, so internally we substract (default) 0.01 from every Second3-Value.
					SCR_Second3Diff = 0.01 results in one object is visible from 0 to 9.99 and the next from 10 to 19.99.
					Maybe raise some trouble behind the final frame, so maybe set finalSecond to 29.99 instead of 30 also.
					Beware: This Value will be substracted, so +0.01 results in Second3-Value 9.99.

SCR_Second			The actual second OF THE ACTUAL SCENE, thats gets rendered or simulated.
					If you have 3 Scenes with 10 Seconds and you render second 16.5, the value is 6.5, not 16.5!
					
SCR_SecondOffset	The FIRST SECOND of the actual Scene, thats gets rendered or simulated.
					If you have 3 Scenes with 10 Seconds and you render second 16.5, the value is 10, not 16.5 or 6.5!
					This is needed so that you can handle every Scene as its were the first, starting at second 0.
					In every macro SCR_SetNumber, SetVector and so on this value gets added internally to the given value.
					This is nice if you later cut the previous scene about a few seconds or insert/delete/move complete scenes
					Than you dont have to adjust all other scenes following.

	
*/// --------------------------------------------------------------------------------------------------------------------------
#ifndef (SCR_Data)				#declare SCR_Data				= array[200][3];		#end	// array of array of string
#ifndef (SCR_DataIndex)			#declare SCR_DataIndex			= 0;					#end	// next to use index of SCR_Data

#ifndef (SCR_Register)			#declare SCR_Register			= array[100][4];		#end	// array of array of strings
#ifndef (SCR_RegisterIndex)		#declare SCR_RegisterIndex		= 0;					#end	// next to use index of SCR_Register

#ifndef (SCR_Scenes)			#declare SCR_Scenes				= array[20][4];			#end	// array of array of strings
#ifndef (SCR_ScenesIndex)		#declare SCR_ScenesIndex		= 0;					#end	// next to use index of SCR_Register
#ifndef (SCR_ScenesAct)			#declare SCR_ScenesAct			= 0;					#end	// 
#ifndef (SCR_ScenesActSec)		#declare SCR_ScenesActSec		= 0;					#end	// 
#ifndef (SCR_ScenesIndexInUse)	#declare SCR_ScenesIndexInUse	= 0;					#end	// 

#ifndef (SCR_FPS)				#declare SCR_FPS				= 10;					#end	// Integer

#ifndef (SCR_SecondStart)		#declare SCR_SecondStart		= 0.0;					#end	// float
#ifndef (SCR_SecondEnd)			#declare SCR_SecondEnd			= 10.0;					#end	// float
#ifndef (SCR_SecondFreeze)		#declare SCR_SecondFreeze		= 10.0;					#end	// float

#ifndef (SCR_SecondEndReal)		#declare SCR_SecondEndReal		= 0.0;					#end	// float! Will be calculated.

#ifndef (SCR_RunSimulation)		#declare SCR_RunSimulation		= false;				#end	// true/false or 0/1 *** not in use at the moment.

#ifndef (SCR_Second2Diff)		#declare SCR_Second2Diff		= 0.00001;				#end	// 0 or 0.01
#ifndef (SCR_Second3Diff)		#declare SCR_Second3Diff		= 0.00001;				#end	// 0 or 0.01
#ifndef (SCR_SecondEndDiff)		#declare SCR_SecondEndDiff		= 0.00001;				#end	// 0 or 0.01

#ifndef (SCR_Second)			#declare SCR_Second				= 0;					#end	// float, dont change!
#ifndef (SCR_SecondOffset)		#declare SCR_SecondOffset		= 0;					#end	// float, dont change!




//-----------------------------------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------------------------

#macro SCR_Start()


	SCR_Init () //Initialize some things, especially register the scenes.
	
	//------------------------------------------------------------------
	// If CLOCK IS NOT RUNNING, start simulation to check Variables.
	// Some Variables and their value will be send to the terminal.
	// Also one Picture will be created (that one of the freeze-Second)
	//------------------------------------------------------------------
	#if (clock_on = 0)
		//Hier muss ich noch bei, von wegen Steuerung Frames, Seconds, Ini, usw.
		//Oder in den Makro-Aufruf verschieben, damit von außen steuerbar.
		#local mStart	=  SCR_SecondStart;
		#local mEnd		=  SCR_SecondEnd;
		#local mStep	=  1 / SCR_FPS;

		//#if (mEnd > SCR_SecondEndReal - SCR_SecondEndDiff)
		//	#local mEnd = SCR_SecondEndReal - SCR_SecondEndDiff;
		//#end
		
		
		
		#if (SCR_RunSimulation)
			#for (SCR_Second, mStart, mEnd, mStep)
				#if ( 1 )
					//MA_Terminal ("----------------------------------------------------------------------")
					MA_Terminal_F("----- Simulated Second running", SCR_Second)
					//MA_Terminal ("-  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -")
				#end
				//Start Timeline as simulation, thereafter reset Data-Array and the Variable that contains its index.
				SCR_Timeline (1)
				#declare SCR_Data		= array[100][3];
				#declare SCR_DataIndex	= 0;
				
			#end
		#end
		
		//Show one picture of the specified time, set by SCR_SecondFreeze
		MA_Terminal_F("----- Create Picture for Second", SCR_SecondFreeze)
		#declare SCR_Second			= SCR_SecondFreeze;
		#declare SCR_SecondOffset	= 0;
		SCR_Timeline (0)
			
		MA_Terminal ("*** Thats it, simulation completed successfully.")

	//------------------------------------------------------------------
	//If the clock is running.
	//In this case create pictures (every time one pictures)
	//------------------------------------------------------------------
	#else
		
		#declare SCR_Second = frame_number / SCR_FPS + SCR_SecondStart;
		#local mLast = SCR_SecondEnd - SCR_SecondEndDiff;

		//Corrects the last frame (manually set by you) if the value calculated by SCR_RegisterScene is smaller
		#if (mLast > SCR_SecondEndReal - SCR_SecondEndDiff)
			#local mLast = SCR_SecondEndReal - SCR_SecondEndDiff;
		#end

		//If the latest Second was done (at the call before), end the job.
		//#if (SCR_Second > SCR_SecondEnd - SCR_SecondEndDiff)			//Hiermit liefs
		#if (SCR_Second > mLast)
		//#if (SCR_Second > SCR_SecondEndReal - SCR_SecondEndDiff)
		
			//To produce an error is one way to end the runtime of an endless animation-rendering,
			//despite the value of Final_Frame isnt reached (set in INI or by +KFF in cmd)
			//That has to happen BEFORE the call of SCR_Timeline(),
			//because an error after it will suck the last rendered Image and will not save it to the disc.
			#error "This_Error_was_raised_to_finalize_the_animation_after_the_last_frame___See_macro_SCR_Start___Everything_is_fine___Greetings__Ma"
		#end
		
		//MA_Terminal ("----------------------------------------------------------------------")
		MA_Terminal_F("----------------------------- REAL Second running", SCR_Second)
		//MA_Terminal ("-  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -")
		SCR_Timeline (0)
		
	#end
#end



//-----------------------------------------------------------------------------------------------------------------------------
// Register all of your scenes by name and duration in seconds. Example:
// SCR_RegisterScene ( "beginning", 20)
// SCR_RegisterScene ( "end",		15)
//-----------------------------------------------------------------------------------------------------------------------------

#macro SCR_RegisterScene ( mName, mDuration )

	//Check if theres free space in the array
	#if (SCR_ScenesIndex > dimension_size(SCR_Scenes,1) - 1)
		#error concat("*** Screenplay-ERROR ***: Dimensions of variable SCR_Scenes insufficient, increase it at declaration in ma_screenplay.inc or your own file. Current-Value: " str(dimension_size(SCR_Scenes,1),0,0))
	#end

	#declare SCR_Scenes [SCR_ScenesIndex][0]	= mName;									//0 = Name
	#declare SCR_Scenes [SCR_ScenesIndex][1]	= Str(mDuration);							//1 = Duration
	#declare SCR_Scenes [SCR_ScenesIndex][2]	= Str(ceil(SCR_ScenesActSec));				//2 = Second start, absolute
	#declare SCR_ScenesActSec					= SCR_ScenesActSec + mDuration;
	#declare SCR_Scenes [SCR_ScenesIndex][3]	= Str(SCR_ScenesActSec - SCR_SecondEndDiff);//3 = Second end, absolute
	
	#declare SCR_SecondEndReal					= SCR_SecondEndReal + mDuration;			//Adds all duration of all scenes together to calculate last second

	#declare SCR_ScenesIndex					= SCR_ScenesIndex + 1;						//Prepare for the next scene to register

	//Debug-Output to the terminal
	#if (0)
		MA_Terminal_F("SCR_ScenesIndex  ", SCR_ScenesIndex)
		MA_Terminal_F("SCR_SecondEndReal", SCR_SecondEndReal)
		MA_Terminal_S("Name             ", SCR_Scenes [SCR_ScenesIndex-1][0])
		MA_Terminal_S("Duration         ", SCR_Scenes [SCR_ScenesIndex-1][1])
		MA_Terminal_S("SecStart         ", SCR_Scenes [SCR_ScenesIndex-1][2])
		MA_Terminal_S("SecEnd           ", SCR_Scenes [SCR_ScenesIndex-1][3])
	#end
#end



//-----------------------------------------------------------------------------------------------------------------------------
// Returns the name of the actual scene that gets rendered.
// Use this in the second-part of you timeline where to create the actors/objects
//-----------------------------------------------------------------------------------------------------------------------------
#macro SCR_GetSceneName ()
	SCR_Scenes [SCR_ScenesAct][0]
#end

//-----------------------------------------------------------------------------------------------------------------------------
// Set or choose the scene you like to describe, in the first part of your timeline where you define when to do what.
// With this at the begin of every scene you can use seconds 0 to x in every scene (instead of 10 to 20 or so in 2nd scene)
// For Second0, Second1 and so on.
//-----------------------------------------------------------------------------------------------------------------------------

#macro SCR_ChooseScene	( mName ) 	SCR_SetScene( mName ) #end
#macro SCR_SetScene		( mName )

	//#local mIndex = MA_Array_FindRow ( mName, SCR_Scenes, 0); //Umgebaut am 22.10.24
	#local mIndex = MA_ArrayString2D_FindRow ( SCR_Scenes, mName, 0);

	#if (mIndex >= 0)
		#declare SCR_ScenesIndexInUse = mIndex;
		//MA_Terminal_F("SCR_ScenesIndexInUse",SCR_ScenesIndexInUse)
	#else
		//Scene-name not found. Error/Warning
	#end
	#declare SCR_SecondOffset = val(SCR_Scenes[SCR_ScenesIndexInUse][2]);
	
	#if (SCR_Second >= val(SCR_Scenes[SCR_ScenesIndexInUse][2]) & SCR_Second < val(SCR_Scenes[SCR_ScenesIndexInUse][3]))
		#declare SCR_ScenesAct = SCR_ScenesIndexInUse;		//MA_Terminal_F("SCR_ScenesAct", SCR_ScenesAct)

	#end
	
	//Debug-Output to the terminal
	#if (0)
		MA_Terminal_S("Defines for Scene", SCR_Scenes[SCR_ScenesIndexInUse][0])
		MA_Terminal_S("Duration", SCR_Scenes[SCR_ScenesIndexInUse][1])
	#end
#end



//-----------------------------------------------------------------------------------------------------------------------------
// This is only a fast helper to reset some values at the beginning of each scene.
// Sets most of them = 0 and a few to the last Second of the scene, defined by SCR_RegisterScene()
// For Examples see the tutorial.
//-----------------------------------------------------------------------------------------------------------------------------
#macro SCR_FillSceneVars	(mSecondStart, mSecondEnd)
	#local mSecondStart	= 0;
	#local mSecondEnd	= val(SCR_Scenes[SCR_ScenesIndexInUse][1]);
#end
#macro SCR_FillSceneVarsAll	(mSecondStart, mSecondEnd, mSecond0, mSecond1, mSecond2, mSecond3)
	#local mSecondStart	= 0;
	#local mSecondEnd	= val(SCR_Scenes[SCR_ScenesIndexInUse][1]);
	#local mSecond0		= 0;
	#local mSecond1		= 0;
	#local mSecond2		= val(SCR_Scenes[SCR_ScenesIndexInUse][1]);
	#local mSecond3		= val(SCR_Scenes[SCR_ScenesIndexInUse][1]);

	//Debug-Output to the terminal
	#if (0)
		MA_Terminal_F("mSecondStart",	mSecondStart)
		MA_Terminal_F("mSecondEnd  ", 	mSecondEnd)
		MA_Terminal_F("mSecond0    ",	mSecond0)
		MA_Terminal_F("mSecond1    ",	mSecond1)
		MA_Terminal_F("mSecond2    ",	mSecond2)
		MA_Terminal_F("mSecond3    ",	mSecond3)
	#end
#end


//-----------------------------------------------------------------------------------------------------------------------------
//This are internally used functions.
//All Datas from SCR_Set*() have to be prepared before they get saved in the big Data-Array.
//SetMixed und GetMixed added at 29.11.18.
//-----------------------------------------------------------------------------------------------------------------------------

#macro SCR_Array_SetFloat	(mName, mProp, mValue)	SCR_Array_Set (mName, mProp, str(mValue,0,-1))			#end
#macro SCR_Array_SetNumber	(mName, mProp, mValue)	SCR_Array_Set (mName, mProp, str(mValue,0,-1))			#end
//#macro SCR_Array_SetVector	(mName, mProp, mValue)	SCR_Array_Set (mName, mProp, vstr(3, mValue, ",", 0,4))	#end
#macro SCR_Array_SetVector	(mName, mProp, mValue)	SCR_Array_Set (mName, mProp, vstr(3, mValue, ",", 0,5))	#end
#macro SCR_Array_SetString	(mName, mProp, mValue)	SCR_Array_Set (mName, mProp, mValue)					#end
#macro SCR_Array_Set		(mName, mProp, mValue)
	//Check if theres free space in the data-array
	#if (SCR_DataIndex > dimension_size(SCR_Data,1) - 1)
		#error concat("*** Screenplay-ERROR ***: Dimensions of variable SCR_Data insufficient, increase it at declaration in ma_screenplay.inc or your own file. Current-Value: " str(dimension_size(SCR_Data,1),0,0))
	#end
	#declare SCR_Data [SCR_DataIndex][0] = mName;
	#declare SCR_Data [SCR_DataIndex][1] = mProp;
	#declare SCR_Data [SCR_DataIndex][2] = mValue;
	#declare SCR_DataIndex = SCR_DataIndex + 1;
	SCR_Trace("*SET*", mName, mProp, mValue)
#end

//-----------------------------------------------------------------------------------------------------------------------------
//This you can use to get variables inside a scene
//-----------------------------------------------------------------------------------------------------------------------------
#macro SCR_GetFloat		(mName1, mName2, mProp, mDefault) SCR_GetNumber	(mName1, mName2, mProp, mDefault) #end
#macro SCR_GetNumber	(mName1, mName2, mProp, mDefault)
	#local mDefault		= str(mDefault,0,-1);
	#local mNewValue	= SCR_Get (mName1, mName2, mProp, mDefault);
	#local mNewValue	= val(mNewValue);
	mNewValue
#end
#macro SCR_GetVector	(mName1, mName2, mProp, mDefault)
	//#local mDefault		= vstr(3, mDefault, ",", 0,1);
	#local mDefault		= vstr(3, mDefault, ",", 0,5);
	#local mTemp		= SCR_Get (mName1, mName2, mProp, mDefault);
	#local mArray		= MA_String2Array (mTemp, ",", 1);
	#if (dimension_size(mArray,1) = 3)
		#local mNewValue	= <val(mArray[0]), val(mArray[1]), val(mArray[2])>;
	#else
		#local mNewValue	= <val(mArray[0]), val(mArray[1]), val(mArray[2]), val(mArray[3])>; //Not yet implemented/tested
	#end
	mNewValue
#end
#macro SCR_GetString	(mName1, mName2, mProp, mDefault)
	#local mNewValue	= SCR_Get (mName1, mName2, mProp, mDefault);
	mNewValue
#end




#macro SCR_GetMixed_Float	(mName1, mName2, mProp, mIndex, mDefault)
	SCR_GetMixed_Number(mName1, mName2, mProp, mIndex, mDefault)
#end
#macro SCR_GetMixed_Number	(mName1, mName2, mProp, mIndex, mDefault)
	SCR_GetMixed_Base("float", mName1, mName2, mProp, mIndex, mIndex, mDefault)
#end
#macro SCR_GetMixed_String	(mName1, mName2, mProp, mIndex, mDefault)
	SCR_GetMixed_Base("string", mName1, mName2, mProp, mIndex, mIndex, mDefault)
#end
#macro SCR_GetMixed_Vector	(mName1, mName2, mProp, mIndexFirst, mIndexLast, mDefault)
	SCR_GetMixed_Base("vector", mName1, mName2, mProp, mIndexFirst, mIndexLast, mDefault)
#end
#macro SCR_GetMixed_ArrayString	(mName1, mName2, mProp, mIndexFirst, mIndexLast, mDefault)
	SCR_GetMixed_Base("array-string", mName1, mName2, mProp, mIndexFirst, mIndexLast, mDefault)
#end

#macro SCR_GetMixed_ArrayFloat	(mName1, mName2, mProp, mIndexFirst, mIndexLast, mDefault)
	SCR_GetMixed_Base("array-float", mName1, mName2, mProp, mIndexFirst, mIndexLast, mDefault)
#end

//Returns a String-Array all the time
#macro SCR_GetMixed_Base	(mType, mName1, mName2, mProp, mIndexFirst, mIndexLast, mDefault)
	#local mName		= SCR_GetName(mName1, mName2)			//only for SCR_Trace, see below
	#local mTempString	= SCR_Get (mName1, mName2, mProp, "");	//gets the string, whatever var-type it will be (here its a string)
	#if (mTempString = "")
		//MA_Terminal("empty mixed-value")
		#local mNewValue = mDefault;
	#else
		#local mTempArray	= MA_String2Array (mTempString, ",", 1)
		//Dimension-size is 1-based, mIndex 0-based
		//#if (dimension_size(mTempArray,1) > mIndexLast)	
		//#if (dimension_size(mTempArray,1) > mIndexFirst & dimension_size(mTempArray,1) > mIndexLast)
		//#if (dimension_size(mTempArray,1) > mIndexFirst & ( dimension_size(mTempArray,1) > mIndexLast | mIndexLast = -1))
		#if (dimension_size(mTempArray,1) > mIndexFirst & ( dimension_size(mTempArray,1) > mIndexLast))	//last correction 23.12.2019
			#if (mType = "float")
				#local mNewValue	= val(mTempArray[mIndexFirst]);
				SCR_Trace(" Getf", mName, mProp, Str(mNewValue))
			#elseif (mType = "string")
				#local mNewValue	= mTempArray[mIndexFirst];
				SCR_Trace(" Gets", mName, mProp, mNewValue)
			#elseif (mType = "vector")
				#local mNewValue	= MA_Array_Cut(mTempArray, mIndexFirst, mIndexLast, -1);
				#local mNewValue	= MA_ArrayString2Vector(mNewValue);
				SCR_Trace(" Getv", mName, mProp, MA_Vector2String(mNewValue))
			#elseif (mType = "array-string")
				#local mNewValue	= MA_Array_Cut(mTempArray, mIndexFirst, mIndexLast, -1);
				SCR_Trace(" Geta", mName, mProp, mNewValue)
			#elseif (mType = "array-float")
				//Here iam missing the function MA_ArrayStrin2ArrayFloat() :/
				#local mCuttedArray	= MA_Array_Cut(mTempArray, mIndexFirst, mIndexLast, -1);
				#local mNewValue = array [ dimension_size(mCuttedArray,1)];				//MA_Terminal_F("Cutted-Array-Dims", dimension_size(mCuttedArray,1))
				#for (mIndexCutted, 0, dimension_size(mCuttedArray,1) - 1)				//MA_Terminal_F("New-Val", val(mCuttedArray[mIndexCutted]))
					#local mNewValue[mIndexCutted] = val(mCuttedArray[mIndexCutted]);
				#end
				SCR_Trace(" Geta", mName, mProp, MA_ArrayFloat2String(mNewValue, ","))
			#end
		#else
			//Maybe the prop exists but is totally empty or has not enough dimensions (comma-separated values, as ordered with mIndexLast)
			#local mNewValue = mDefault;
		#end
	#end
	mNewValue
#end


#macro SCR_Get	(mName1, mName2, mProp, mDefault)
	
	//Hier noch Prop checken, ob Slash drin.
	//Wenn ja, dann teilen und mit allen Items des Arrays diese Funktion aufrufen.
	
	#if		( strlen(mName1) > 0 & strlen(mName2) = 0)
		#local mName = mName1;
	#elseif	( strlen(mName1) = 0 & strlen(mName2) > 0)
		#local mName = mName2;
	#else
		#local mName = concat (mName1, "/", mName2);
	#end
	#local mValue = mDefault;
	
	#if (mName = "")
		//Error/Warning
		#local mValue = mDefault;
	#elseif (mProp = "")
		//Error/Warning
		#local mValue = mDefault;
	#else
		//Looks good, search whole Array for Name and Prop
		//#for (mIndex, 0, dimension_size(SCR_Data,1) - 1)
		#for (mIndex, 0, SCR_DataIndex - 1)
			//MA_Terminal_F("Index", mIndex)
			#local mArrayName	= SCR_Data[mIndex][0];
			#local mArrayProp	= SCR_Data[mIndex][1];
			#local mArrayValue	= SCR_Data[mIndex][2];
			#if (mArrayName = "")
				//No Name means from here on only empty Array-Rows. End the loop.
				#break
			#elseif (strlwr(mArrayName) = strlwr(mName) & strlwr(mArrayProp) = strlwr(mProp))
				//Row found. After that dont break loop, because the last inserted Value will overwrite them before (last one rulez, i dont like the early birds)
				#local mValue = mArrayValue;
			#end
		#end
		SCR_Trace(" Get ", mName, mProp, mValue)
	#end

	mValue
#end



//-----------------------------------------------------------------------------------------------------------------------------
//Small helpers you can use in the definition-phase and later check in the creation-phase if existing.
//But most of the time you likely will use SCR_GetSceneName()
//-----------------------------------------------------------------------------------------------------------------------------

#macro SCR_SetMarker (mName, xSecondStart, xSecondEnd)

	#local mSecondNow	= SCR_Second;
	#local mSecondStart	= xSecondStart + SCR_SecondOffset;
	#local mSecondEnd	= xSecondEnd   + SCR_SecondOffset - SCR_Second3Diff;
	#if (mSecondNow >= mSecondStart & mSecondNow <= mSecondEnd)
		SCR_Array_SetNumber (mName, "INTERNAL-marker", 1)
	#end
#end
#macro SCR_GetMarker (mName)
	#local mNewValue	= SCR_GetNumber	(mName, "", "INTERNAL-marker", 0);
	mNewValue
#end

//Deprecated, compatibility to old names, will vanish in 2025.
#macro SCR_SetExist (mName, xSecondStart, xSecondEnd)
	MA_Deprecated ("SCR_SetExist", "SCR_SetMarker")
	SCR_SetMarker (mName, xSecondStart, xSecondEnd)
#end
#macro SCR_GetExist (mName)
	MA_Deprecated ("SCR_GetExist", "SCR_GetMarker")
	SCR_GetMarker (mName)
#end


//-----------------------------------------------------------------------------------------------------------------------------
//Endstate 0 = uses mValueStart after mSecond2. 1 = uses mValueEnd till mSecond3
//We need xSecond and mSecond because povray returns the changed value back to the macro-caller (and we dont like that)
//-----------------------------------------------------------------------------------------------------------------------------
#macro SCR_SetFloat (mName, mProp, mValueStart, mValueEnd, mSlope, xSecond0, xSecond1, xSecond2, xSecond3, mEndState)
	SCR_SetNumber (mName, mProp, mValueStart, mValueEnd, mSlope, xSecond0, xSecond1, xSecond2, xSecond3, mEndState)
#end
#macro SCR_SetNumber (mName, mProp, mValueStart, mValueEnd, mSlope, xSecond0, xSecond1, xSecond2, xSecond3, mEndState)

	#local mSecondNow = SCR_Second;
	#local mSecond0 = xSecond0 + SCR_SecondOffset;
	#local mSecond1 = xSecond1 + SCR_SecondOffset;
	#local mSecond2 = xSecond2 + SCR_SecondOffset - SCR_Second2Diff;
	#local mSecond3 = xSecond3 + SCR_SecondOffset - SCR_Second3Diff;
	#if ((mSecondNow >= mSecond0 & mSecondNow <= mSecond3) | (mSecondNow >= mSecond0 & mSecond3 = 0))
		//We are inside mSecond0 and mSecond3, so start the evaluation
		#if (mSecondNow < mSecond1)
			#local mValue	= mValueStart;
		#elseif (mSecondNow >= mSecond1 & mSecondNow <= mSecond2)
			//Event ongoing
			#local mTimeFactor	= 1 / (mSecond2 - mSecond1) * (mSecondNow - mSecond1);
			#local mTimeFactor	= MA_GetSlope_ByString ( mTimeFactor, mSlope );
			#local mValue		= mValueStart + ((mValueEnd - mValueStart) * mTimeFactor);
		#else
			//Event happend, finish-Value
			#if (mEndState = 1)
				//Last Value will be the one for the time after
				#local mValue	= mValueEnd;
			#else
				//Go back to the first Value
				#local mValue	= mValueStart;
			#end
		#end
		SCR_Array_SetNumber	(mName, mProp, mValue)
	#else
		//The actual time is not inside Second0 and Second3, so ignore it and end this macro
		//If the actual Time is bigger than Second3, also the modifyed value will disapear, as changes never were made
	#end

#end

#macro SCR_SetVector (mName, mProp, mValueStart, mValueEnd, mSlope, xSecond0, xSecond1, xSecond2, xSecond3, mEndState)
	#local mSpline = spline {
		linear_spline
		0.00, mValueStart
		1.00, mValueEnd
	}
	SCR_SetSpline (mName, mProp, mValueStart, mSpline, mSlope, xSecond0, xSecond1, xSecond2, xSecond3, mEndState)
#end

#macro SCR_SetSpline (mName, mProp, mValueStart, mSpline, mSlope, xSecond0, xSecond1, xSecond2, xSecond3, mEndState)
	
	#local mSecondNow = SCR_Second;
	#local mSecond0 = xSecond0 + SCR_SecondOffset;
	#local mSecond1 = xSecond1 + SCR_SecondOffset;
	#local mSecond2 = xSecond2 + SCR_SecondOffset - SCR_Second2Diff;
	#local mSecond3 = xSecond3 + SCR_SecondOffset - SCR_Second3Diff;
	#if ((mSecondNow >= mSecond0 & mSecondNow <= mSecond3) | (mSecondNow >= mSecond0 & mSecond3 = 0))
		//We are inside mSecond0 and mSecond3, so start the evaluation
		#if (mSecondNow < mSecond1)
			#local mValue	= mValueStart;
		#elseif (mSecondNow >= mSecond1 & mSecondNow <= mSecond2)
			//Event ongoing
			#local mTimeFactor	= 1 / (mSecond2 - mSecond1) * (mSecondNow - mSecond1);
			#local mTimeFactor	= MA_GetSlope_ByString ( mTimeFactor, mSlope );
			#local mValue		= mSpline (mTimeFactor);
			//MA_Terminal_F("mTimeFactor in SetSpline", mTimeFactor)
			
		#else
			//Event happend, finish-Value
			#if (mEndState = 1)
				//Last Value will be the one for the time after
				#local mValue	= mSpline(1);
			#else
				//Go back to the first Value
				#local mValue	= mSpline(0);
			#end
		#end
		SCR_Array_SetVector	(mName, mProp, mValue)

	#else
		//The actual time is not inside Second0 and Second3, so ignore it and end this macro
		//If the actual time is bigger than Second3, also the modifyed value will disapear, as changes never were made
	#end

#end

#macro SCR_SetString (mName, mProp, mValue1, mValue2, mValue3, xSecond0, xSecond1, xSecond2, xSecond3)

	//mSlope not in use at the Moment
	#local mSecondNow = SCR_Second;
	#local mSecond0 = xSecond0 + SCR_SecondOffset;
	#local mSecond1 = xSecond1 + SCR_SecondOffset;
	#local mSecond2 = xSecond2 + SCR_SecondOffset - SCR_Second2Diff;
	#local mSecond3 = xSecond3 + SCR_SecondOffset - SCR_Second3Diff;
	#if ((mSecondNow >= mSecond0 & mSecondNow <= mSecond3) | (mSecondNow >= mSecond0 & mSecond3 = 0))
		//We are inside mSecond0 and mSecond3, so start the evaluation
		#if (mSecondNow < mSecond1)
			#local mValue	= mValue1;
		#elseif (mSecondNow >= mSecond1 & mSecondNow <= mSecond2)
			//Event ongoing
			#local mValue	= mValue2;
		#else
			//Event happend, finish-Value
			#local mValue	= mValue3;
		#end
		SCR_Array_SetString	(mName, mProp, mValue)
	#else
		//The actual time is not inside Second0 and Second3, so ignore it and end this macro
		//If the actual Time is bigger than Second3, also the modifyed value will disapear, as changes never were made
	#end

#end


#macro SCR_SetMixed (mName, mProp, mValueStart, mValueEnd, mSlope, xSecond0, xSecond1, xSecond2, xSecond3, mEndState)
	SCR_SetOrSumMixed ("set", mName, mProp, mValueStart, mValueEnd, mSlope, xSecond0, xSecond1, xSecond2, xSecond3, mEndState)
#end
#macro SCR_SumMixed (mName, mProp, mValueStart, mValueEnd, mSlope, xSecond0, xSecond1, xSecond2, xSecond3, mEndState)
	SCR_SetOrSumMixed ("sum", mName, mProp, mValueStart, mValueEnd, mSlope, xSecond0, xSecond1, xSecond2, xSecond3, mEndState)
#end
#macro SCR_SetOrSumMixed (mType, mName, mProp, mValueStart, mValueEnd, mSlope, xSecond0, xSecond1, xSecond2, xSecond3, mEndState)


	//mSlope not in use at the Moment
	#local mSecondNow = SCR_Second;
	#local mSecond0 = xSecond0 + SCR_SecondOffset;
	#local mSecond1 = xSecond1 + SCR_SecondOffset;
	#local mSecond2 = xSecond2 + SCR_SecondOffset - SCR_Second2Diff;
	#local mSecond3 = xSecond3 + SCR_SecondOffset - SCR_Second3Diff;
	#if ((mSecondNow >= mSecond0 & mSecondNow <= mSecond3) | (mSecondNow >= mSecond0 & mSecond3 = 0))
		//We are inside mSecond0 and mSecond3, so start the evaluation
		#if (mSecondNow < mSecond1)
			#local mValue	= mValueStart;
		#elseif (mSecondNow >= mSecond1 & mSecondNow <= mSecond2)
			//Event ongoing MA_Terminal("Event ongoing")  MA_Terminal_S("Value before", mValueStart)
			#local mValue = SCR_SetMixed_CalculateValue(mValueStart, mValueEnd, mSlope, mSecondNow, mSecond1, mSecond2);
		#else
			//Event happend, finish-Value
			#local mValue	= mValueEnd;
		#end
		
		
		#if (mType = "set")
			SCR_Array_SetString	(mName, mProp, mValue)
		#else
		
			//Look for an existing entry or name and prop
			//if existing, add both values and update the new value in the array with the same index
			//#local mIndex = MA_Array_FindRow_Multi ( SCR_Data, mMyName, mProp, 0, 1, 0, SCR_DataIndex-1);	//Umgebaut 22.10.24
			#local mIndex = MA_ArrayString2D_FindRow_Multi ( SCR_Data, mMyName, mProp, 0, 1, 0, SCR_DataIndex-1);
			
			
			//If one was found, sum both together. If not, use unaltered mValue
			#if (mIndex >= 0)
				//MA_Terminal_F("awoef mIndex", mIndex)
				//MA_Error ("SCR_SetSoulAction", "bingo")
				#local mValueExisting = SCR_Data[mIndex][2];
				//MA_Terminal_S("SCR_SetOrSumMixed - mValue", mValue)
				//MA_Terminal_S("SCR_SetOrSumMixed - mValueExisting", mValueExisting)
				
				#local mValue = MA_Mixed_SumValues (mValue, mValueExisting)
			#end
		
			SCR_Array_SetString	(mName, mProp, mValue)
		
		
		#end
		
	#else
		//The actual time is not inside Second0 and Second3, so ignore it and end this macro
		//If the actual Time is bigger than Second3, also the modifyed value will disapear, as changes never were made
	#end

#end


/*
Split both Value-Arrays in String, walk through arrays.
Handle both as floats and calculate the new value.
Save them in an (string-formatted) array, concat the array-items to a new string

HIER FEHLT GLAUB ICH NOCH DIE SLOPE-ANGABE.
Außerdem wohl umbenennen, da auch für Multi benötigt.

*/

#macro SCR_SetMixed_CalculateValue(mValueStart, mValueEnd, mSlope, mSecondNow, mSecondStart, mSecondEnd)

	#local mTimeFactor	= 1 / (mSecondEnd - mSecondStart) * (mSecondNow - mSecondStart);
	#local mTimeFactor	= MA_GetSlope_ByString ( mTimeFactor, mSlope );
	#local mArrayStart	= MA_String2Array (mValueStart, ",", 1)
	#local mArrayEnd	= MA_String2Array (mValueEnd  , ",", 1)
	#local mArrayResult	= array[dimension_size(mArrayStart,1)]; //Only to get an array with the same dimensions, NOT float formatted
	
	#for (mIndex, 0, dimension_size(mArrayStart,1) - 1)
		#local mStart	= val(mArrayStart[mIndex]);
		#local mEnd		= val(mArrayEnd  [mIndex]);
		#local mArrayResult[mIndex]	= Str(mStart + ((mEnd - mStart) * mTimeFactor));
	#end
	#local mValue = MA_ArrayString2String(mArrayResult, ",");
	mValue
#end












//-----------------------------------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------------------------



#macro SCR_TraceAddName(mName)
	//DoNothingLoop()
#end

#macro SCR_Trace(mJob, mName, mProp, mValue)
	#if (clock_on = 0 & 1)
		#local mMessage = concat (mJob, " => ", "N: ", mName, "; P: ", mProp, "; V: ", mValue);
		MA_Terminal(mMessage)
	#end

#end




/*
	Simply concats 2 names
	So "bingo" and "bongo" becomes "bingo/bongo", but "bingo" and "" does NOT become "bingo/", instead "bingo".
	Will only save time and lines, no one needs to concat names in its own macro.
*/
#macro SCR_GetName(mName1, mName2)
	#local mSeparator = "/";
	#if (mName1 = "" | mName2 = "")
		#local mSeparator = "";
	#end
	concat (mName1, mSeparator, mName2)
#end

//Shortcut for SCR_GetName()
#macro MA_CC (mName1, mName2)
	SCR_GetName(mName1, mName2)
#end
#macro MA_CCC (mName, mArray)
	#for (mIndex, 0, dimension_size(mArray,1) - 1)
		#local mArray[mIndex] = SCR_GetName(mName, mArray[mIndex]);
	#end
	mArray
#end















//-----------------------------------------------------------------------------------------------------------------------------
//This is the end of the very first If (that checks if this file was included before)
//Never delete it, or else ...
	#version MA_Screenplay_Temp;
#end
//-----------------------------------------------------------------------------------------------------------------------------
